'use strict';

const db = require.main.require('./src/database');
const topics = require.main.require('./src/topics');
const user = require.main.require('./src/user');
const privileges = require.main.require('./src/privileges');
const events = require.main.require('./src/events');

const plugin = module.exports;

plugin.init = async (params) => {
	const router = params.router;
	const hostMiddleware = params.middleware;
	const middleware = require('./lib/middleware');
	const controllers = require('./lib/controllers');
	// const hostControllers = params.controllers;

	router.post('/api/topics/:tid/bury', hostMiddleware.authenticateRequest, hostMiddleware.ensureLoggedIn, middleware.ensurePrivilege, middleware.logBurial, controllers.bury);
	router.delete('/api/topics/:tid/bury', hostMiddleware.authenticateRequest, hostMiddleware.ensureLoggedIn, middleware.ensurePrivilege, middleware.logBurial, controllers.exhume);

	// Add new audit log type
	events.types.push('topic-buried');
};

plugin.addPrivilege = async function (privileges) {
	privileges.push((this.hook === 'filter:privileges.groups.list' ? 'groups:' : '') + 'buryTopics');
	return privileges;
};

plugin.addPrivilegeLabel = async (labels) => {
	labels.push({ name: 'Bury Topics' });
	return labels;
};

plugin.bury = async (tid, type) => {
	// type 1: can't see if never visited before, type 2: can't see if not replied
	await topics.setTopicField(tid, 'buried', type);
};

plugin.addTopicTool = async (data) => {
	const isPrivileged = await privileges.categories.can('buryTopics', data.topic.cid, data.uid);
	if (isPrivileged) {
		data.tools.push({
			class: 'bury-topic',
			icon: 'fa-eye-slash',
			title: 'Bury Topic',
		});
	}

	return data;
};

plugin.filterBuried = async (data) => {
	if (!data.topics.length) {
		return data;
	}

	const checks = await plugin.check(data.uid, data.topics.map(topic => topic.tid));
	data.topics = data.topics.filter((topic, idx) => checks[idx]);
	return data;
};

plugin.filterBuriedTids = async (data) => {
	if (!data.tids.length) {
		return data;
	}

	const checks = await plugin.check(data.params.uid, data.tids);
	data.tids = data.tids.filter((tid, idx) => checks[idx]);
	return data;
};

plugin.filterBuriedUnreads = async (data) => {
	if (!data.tids.length) {
		return data;
	}

	const checks = await plugin.check(data.uid, data.tids);
	data.tids = data.tids.filter((tid, idx) => checks[idx]);
	data.tidsByFilter[data.filter] = data.tidsByFilter[data.filter].filter((tid, idx) => checks[idx]);
	data.counts[data.filter] = data.tidsByFilter[data.filter].length;

	return data;
};

plugin.filterBuriedTeasers = async (data) => {
	if (!data.teasers.length) {
		return data;
	}

	const checks = await plugin.check(data.uid, data.teasers.map(teaser => (teaser || {}).tid));
	data.teasers = data.teasers.map((teaser, idx) => {
		if (!checks[idx]) {
			return false;
		}

		return teaser;
	});
	return data;
};

plugin.filterBuriedPostSummaries = async (data) => {
	if (!data.posts.length) {
		return data;
	}

	const checks = await plugin.check(data.uid, data.posts.map(post => post && post.tid));
	data.posts = data.posts.filter((post, idx) => post && checks[idx]);
	return data;
};

plugin.check = async (uid, tids) => {
	// Arguments read as: can uid access tid?
	const states = (await topics.getTopicsFields(tids, ['buried'])).map(obj => parseInt(obj.buried || 0, 10));
	const isAdminOrGmod = await user.isAdminOrGlobalMod(uid);
	const hasViewed = await db.isSortedSetMembers('uid:' + uid + ':tids_read', tids);
	const hasReplied = await db.isMemberOfSortedSets(tids.map(tid => 'tid:' + tid + ':posters'), uid);

	// console.log('states', states);
	// console.log('hasViewed', hasViewed);
	// console.log('hasReplied', hasReplied);

	return states.map((state, idx) => {
		switch (state) {
		case 1:
			return isAdminOrGmod || hasViewed[idx];
		case 2:
			return isAdminOrGmod || hasReplied[idx];
		default:
			return true;
		}
	});
};
